/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.net;

import java.net.InetSocketAddress;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.jboss.netty.bootstrap.ClientBootstrap;
import org.jboss.netty.channel.ChannelFuture;
import org.jboss.netty.channel.ChannelHandlerContext;
import org.jboss.netty.channel.ChannelPipeline;
import org.jboss.netty.channel.ChannelPipelineFactory;
import org.jboss.netty.channel.Channels;
import org.jboss.netty.channel.socket.nio.NioClientSocketChannelFactory;

/**
 * Modification of {@link EchoClient} which utilizes Java object serialization.
 */
public abstract class Client extends Netty {

	private final String host;
	private final int port;
	private boolean ready=false;
	protected ChannelHandlerContext context;
	ClientBootstrap bootstrap;
	ExecutorService pool1;
	ExecutorService pool2;
	ChannelFuture connectFuture;
	protected int timeout = 5000;// ms

	public int getTimeout() {
		return timeout;
	}

	public void stop() {
		bootstrap.shutdown();
	}

	public void setTimeout(int timeout) {
		this.timeout = timeout;
	}

	public Client(String host, int port) {
		this.host = host;
		this.port = port;
	}

	public abstract <T> T sendObject(Object object);

	public void run() {
		// Configure the client.
		pool1 = Executors.newCachedThreadPool();
		pool2 = Executors.newCachedThreadPool();
		bootstrap = new ClientBootstrap(new NioClientSocketChannelFactory(
				pool1, pool2));

		// Set up the pipeline factory.
		bootstrap.setPipelineFactory(new ChannelPipelineFactory() {
			public ChannelPipeline getPipeline() throws Exception {
				return Channels.pipeline(getEncoder(), getDecoder(),
						getHandler());
			}
		});
		connectFuture = bootstrap.connect(new InetSocketAddress(host, port));

		connectFuture.awaitUninterruptibly().getChannel();
		// Wait until the connection is closed or the connection attempt fails.
		connectFuture.getChannel().getCloseFuture().awaitUninterruptibly();

		// Shut down thread pools to exit.
		bootstrap.releaseExternalResources();
	}

	public void setReady(boolean ready) {
		this.ready = ready;
	}

	public boolean isReady() {
		return ready;
	}

	public void setChannelHandlerContext(ChannelHandlerContext ctx) {
		this.context = ctx;

	}

}
