/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.net;

import java.net.InetSocketAddress;
import java.util.concurrent.Executors;

import org.jboss.netty.bootstrap.ServerBootstrap;
import org.jboss.netty.channel.Channel;
import org.jboss.netty.channel.ChannelPipeline;
import org.jboss.netty.channel.ChannelPipelineFactory;
import org.jboss.netty.channel.Channels;
import org.jboss.netty.channel.socket.nio.NioServerSocketChannelFactory;
import org.tinygroup.logger.LogLevel;

/**
 * Modification of {@link EchoServer} which utilizes Java object serialization.
 */
public abstract class Server extends Netty {
	ServerBootstrap bootstrap;
	private final int port;
	Channel channel;

	public Server(int port) {
		this.port = port;
	}

	public void stop() {
		logger.logMessage(LogLevel.INFO, "服务器正在停止中，端口:{} ...", port);
		channel.close();
		bootstrap.shutdown();
		logger.logMessage(LogLevel.INFO, "服务器停止完毕，端口:{}。", port);
	}

	public void run() {
		logger.logMessage(LogLevel.INFO, "服务器正在启动中，端口:{} ...", port);
		// Configure the server.
		bootstrap = new ServerBootstrap(new NioServerSocketChannelFactory(
				Executors.newCachedThreadPool(),
				Executors.newCachedThreadPool()));

		// Set up the pipeline factory.
		bootstrap.setPipelineFactory(new ChannelPipelineFactory() {
			public ChannelPipeline getPipeline() throws Exception {
				return Channels.pipeline(getEncoder(), getDecoder(),
						getHandler());
			}
		});

		// Bind and start to accept incoming connections.
		channel = bootstrap.bind(new InetSocketAddress(port));
		logger.logMessage(LogLevel.INFO, "服务器启动完毕，端口:{}。", port);
		// Wait until the connection is closed or the connection attempt fails.

	}

}
